# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_REPLACE_STRUCTURE_METADATA = Metadata(
    id="06b2d63161345b1accca9efe553084791baedc0f.workbench",
    name="cifti-replace-structure",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiReplaceStructureVolumeAllParamsDictNoTag = typing.TypedDict('_CiftiReplaceStructureVolumeAllParamsDictNoTag', {
    "volume": InputPathType,
    "from-cropped": bool,
})
CiftiReplaceStructureVolumeAllParamsDictTagged = typing.TypedDict('CiftiReplaceStructureVolumeAllParamsDictTagged', {
    "@type": typing.Literal["volume-all"],
    "volume": InputPathType,
    "from-cropped": bool,
})
CiftiReplaceStructureVolumeAllParamsDict = _CiftiReplaceStructureVolumeAllParamsDictNoTag | CiftiReplaceStructureVolumeAllParamsDictTagged


_CiftiReplaceStructureLabelParamsDictNoTag = typing.TypedDict('_CiftiReplaceStructureLabelParamsDictNoTag', {
    "structure": str,
    "label": InputPathType,
})
CiftiReplaceStructureLabelParamsDictTagged = typing.TypedDict('CiftiReplaceStructureLabelParamsDictTagged', {
    "@type": typing.Literal["label"],
    "structure": str,
    "label": InputPathType,
})
CiftiReplaceStructureLabelParamsDict = _CiftiReplaceStructureLabelParamsDictNoTag | CiftiReplaceStructureLabelParamsDictTagged


_CiftiReplaceStructureMetricParamsDictNoTag = typing.TypedDict('_CiftiReplaceStructureMetricParamsDictNoTag', {
    "structure": str,
    "metric": InputPathType,
})
CiftiReplaceStructureMetricParamsDictTagged = typing.TypedDict('CiftiReplaceStructureMetricParamsDictTagged', {
    "@type": typing.Literal["metric"],
    "structure": str,
    "metric": InputPathType,
})
CiftiReplaceStructureMetricParamsDict = _CiftiReplaceStructureMetricParamsDictNoTag | CiftiReplaceStructureMetricParamsDictTagged


_CiftiReplaceStructureVolumeParamsDictNoTag = typing.TypedDict('_CiftiReplaceStructureVolumeParamsDictNoTag', {
    "structure": str,
    "volume": InputPathType,
    "from-cropped": bool,
})
CiftiReplaceStructureVolumeParamsDictTagged = typing.TypedDict('CiftiReplaceStructureVolumeParamsDictTagged', {
    "@type": typing.Literal["volume"],
    "structure": str,
    "volume": InputPathType,
    "from-cropped": bool,
})
CiftiReplaceStructureVolumeParamsDict = _CiftiReplaceStructureVolumeParamsDictNoTag | CiftiReplaceStructureVolumeParamsDictTagged


_CiftiReplaceStructureParamsDictNoTag = typing.TypedDict('_CiftiReplaceStructureParamsDictNoTag', {
    "cifti": str,
    "direction": str,
    "volume-all": typing.NotRequired[CiftiReplaceStructureVolumeAllParamsDict | None],
    "label": typing.NotRequired[list[CiftiReplaceStructureLabelParamsDict] | None],
    "metric": typing.NotRequired[list[CiftiReplaceStructureMetricParamsDict] | None],
    "volume": typing.NotRequired[list[CiftiReplaceStructureVolumeParamsDict] | None],
    "action": typing.NotRequired[str | None],
    "discard-unused-labels": bool,
})
CiftiReplaceStructureParamsDictTagged = typing.TypedDict('CiftiReplaceStructureParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-replace-structure"],
    "cifti": str,
    "direction": str,
    "volume-all": typing.NotRequired[CiftiReplaceStructureVolumeAllParamsDict | None],
    "label": typing.NotRequired[list[CiftiReplaceStructureLabelParamsDict] | None],
    "metric": typing.NotRequired[list[CiftiReplaceStructureMetricParamsDict] | None],
    "volume": typing.NotRequired[list[CiftiReplaceStructureVolumeParamsDict] | None],
    "action": typing.NotRequired[str | None],
    "discard-unused-labels": bool,
})
CiftiReplaceStructureParamsDict = _CiftiReplaceStructureParamsDictNoTag | CiftiReplaceStructureParamsDictTagged


def cifti_replace_structure_volume_all(
    volume: InputPathType,
    from_cropped: bool = False,
) -> CiftiReplaceStructureVolumeAllParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume: the input volume.
        from_cropped: the input is cropped to the size of the data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume-all",
        "volume": volume,
        "from-cropped": from_cropped,
    }
    return params


def cifti_replace_structure_volume_all_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiReplaceStructureVolumeAllParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume", None) is None:
        raise StyxValidationError("`volume` must not be None")
    if not isinstance(params["volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `InputPathType`')
    if params.get("from-cropped", False) is None:
        raise StyxValidationError("`from-cropped` must not be None")
    if not isinstance(params["from-cropped"], bool):
        raise StyxValidationError(f'`from-cropped` has the wrong type: Received `{type(params.get("from-cropped", False))}` expected `bool`')


def cifti_replace_structure_volume_all_cargs(
    params: CiftiReplaceStructureVolumeAllParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume-all",
        execution.input_file(params.get("volume", None))
    ])
    if params.get("from-cropped", False):
        cargs.append("-from-cropped")
    return cargs


def cifti_replace_structure_label(
    structure: str,
    label: InputPathType,
) -> CiftiReplaceStructureLabelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure to replace the data of.
        label: the input label file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "label",
        "structure": structure,
        "label": label,
    }
    return params


def cifti_replace_structure_label_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiReplaceStructureLabelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("label", None) is None:
        raise StyxValidationError("`label` must not be None")
    if not isinstance(params["label"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `InputPathType`')


def cifti_replace_structure_label_cargs(
    params: CiftiReplaceStructureLabelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-label",
        params.get("structure", None),
        execution.input_file(params.get("label", None))
    ])
    return cargs


def cifti_replace_structure_metric(
    structure: str,
    metric: InputPathType,
) -> CiftiReplaceStructureMetricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure to replace the data of.
        metric: the input metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "metric",
        "structure": structure,
        "metric": metric,
    }
    return params


def cifti_replace_structure_metric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiReplaceStructureMetricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')


def cifti_replace_structure_metric_cargs(
    params: CiftiReplaceStructureMetricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-metric",
        params.get("structure", None),
        execution.input_file(params.get("metric", None))
    ])
    return cargs


def cifti_replace_structure_volume(
    structure: str,
    volume: InputPathType,
    from_cropped: bool = False,
) -> CiftiReplaceStructureVolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure to replace the data of.
        volume: the input volume.
        from_cropped: the input is cropped to the size of the component.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume",
        "structure": structure,
        "volume": volume,
        "from-cropped": from_cropped,
    }
    return params


def cifti_replace_structure_volume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiReplaceStructureVolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("volume", None) is None:
        raise StyxValidationError("`volume` must not be None")
    if not isinstance(params["volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `InputPathType`')
    if params.get("from-cropped", False) is None:
        raise StyxValidationError("`from-cropped` must not be None")
    if not isinstance(params["from-cropped"], bool):
        raise StyxValidationError(f'`from-cropped` has the wrong type: Received `{type(params.get("from-cropped", False))}` expected `bool`')


def cifti_replace_structure_volume_cargs(
    params: CiftiReplaceStructureVolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume",
        params.get("structure", None),
        execution.input_file(params.get("volume", None))
    ])
    if params.get("from-cropped", False):
        cargs.append("-from-cropped")
    return cargs


class CiftiReplaceStructureOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiReplaceStructureParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def cifti_replace_structure_params(
    cifti: str,
    direction: str,
    volume_all: CiftiReplaceStructureVolumeAllParamsDict | None = None,
    label: list[CiftiReplaceStructureLabelParamsDict] | None = None,
    metric: list[CiftiReplaceStructureMetricParamsDict] | None = None,
    volume: list[CiftiReplaceStructureVolumeParamsDict] | None = None,
    action: str | None = None,
    discard_unused_labels: bool = False,
) -> CiftiReplaceStructureParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti: the cifti to modify.
        direction: which dimension to interpret as a single map, ROW or COLUMN.
        volume_all: replace the data in all volume components.
        label: replace the data in a surface label component.
        metric: replace the data in a surface component.
        volume: replace the data in a volume component.
        action: how to handle conflicts between label keys\
            \
            'ERROR', 'LEFT_SURFACE_FIRST', or 'LEGACY', default 'ERROR', use\
            'LEGACY' to match v1.4.2 and earlier.
        discard_unused_labels: when operating on a dlabel file, drop any unused\
            label keys from the label table.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-replace-structure",
        "cifti": cifti,
        "direction": direction,
        "discard-unused-labels": discard_unused_labels,
    }
    if volume_all is not None:
        params["volume-all"] = volume_all
    if label is not None:
        params["label"] = label
    if metric is not None:
        params["metric"] = metric
    if volume is not None:
        params["volume"] = volume
    if action is not None:
        params["action"] = action
    return params


def cifti_replace_structure_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiReplaceStructureParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti", None) is None:
        raise StyxValidationError("`cifti` must not be None")
    if not isinstance(params["cifti"], str):
        raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `str`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("volume-all", None) is not None:
        cifti_replace_structure_volume_all_validate(params["volume-all"])
    if params.get("label", None) is not None:
        if not isinstance(params["label"], list):
            raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `list[CiftiReplaceStructureLabelParamsDict] | None`')
        for e in params["label"]:
            cifti_replace_structure_label_validate(e)
    if params.get("metric", None) is not None:
        if not isinstance(params["metric"], list):
            raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `list[CiftiReplaceStructureMetricParamsDict] | None`')
        for e in params["metric"]:
            cifti_replace_structure_metric_validate(e)
    if params.get("volume", None) is not None:
        if not isinstance(params["volume"], list):
            raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `list[CiftiReplaceStructureVolumeParamsDict] | None`')
        for e in params["volume"]:
            cifti_replace_structure_volume_validate(e)
    if params.get("action", None) is not None:
        if not isinstance(params["action"], str):
            raise StyxValidationError(f'`action` has the wrong type: Received `{type(params.get("action", None))}` expected `str | None`')
    if params.get("discard-unused-labels", False) is None:
        raise StyxValidationError("`discard-unused-labels` must not be None")
    if not isinstance(params["discard-unused-labels"], bool):
        raise StyxValidationError(f'`discard-unused-labels` has the wrong type: Received `{type(params.get("discard-unused-labels", False))}` expected `bool`')


def cifti_replace_structure_cargs(
    params: CiftiReplaceStructureParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-replace-structure"
    ])
    cargs.append(params.get("cifti", None))
    cargs.append(params.get("direction", None))
    if params.get("volume-all", None) is not None or params.get("label", None) is not None or params.get("metric", None) is not None or params.get("volume", None) is not None:
        cargs.extend([
            *(cifti_replace_structure_volume_all_cargs(params.get("volume-all", None), execution) if (params.get("volume-all", None) is not None) else []),
            *([a for c in [cifti_replace_structure_label_cargs(s, execution) for s in params.get("label", None)] for a in c] if (params.get("label", None) is not None) else []),
            *([a for c in [cifti_replace_structure_metric_cargs(s, execution) for s in params.get("metric", None)] for a in c] if (params.get("metric", None) is not None) else []),
            *([a for c in [cifti_replace_structure_volume_cargs(s, execution) for s in params.get("volume", None)] for a in c] if (params.get("volume", None) is not None) else [])
        ])
    if params.get("action", None) is not None:
        cargs.extend([
            "-label-collision",
            params.get("action", None)
        ])
    if params.get("discard-unused-labels", False):
        cargs.append("-discard-unused-labels")
    return cargs


def cifti_replace_structure_outputs(
    params: CiftiReplaceStructureParamsDict,
    execution: Execution,
) -> CiftiReplaceStructureOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiReplaceStructureOutputs(
        root=execution.output_file("."),
    )
    return ret


def cifti_replace_structure_execute(
    params: CiftiReplaceStructureParamsDict,
    runner: Runner | None = None,
) -> CiftiReplaceStructureOutputs:
    """
    REPLACE DATA IN A STRUCTURE IN A CIFTI FILE.
    
    This is a fairly low-level command, you probably want to use
    -cifti-create-dense-from-template instead.
    
    You must specify at least one of -metric, -label, -volume, or -volume-all
    for this command to do anything. Input volumes must line up with the output
    of -cifti-separate. For dtseries/dscalar, use COLUMN, and if your dconn
    matrix will be fully symmetric, COLUMN is more efficient. The -volume-all
    option must not be specified when using a -volume option. A -metric option
    must not be specified when using a -label option, and is not recommended on
    a label-type cifti file. For each <structure> argument, use one of the
    following strings:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiReplaceStructureOutputs`).
    """
    cifti_replace_structure_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_REPLACE_STRUCTURE_METADATA)
    params = execution.params(params)
    cargs = cifti_replace_structure_cargs(params, execution)
    ret = cifti_replace_structure_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_replace_structure(
    cifti: str,
    direction: str,
    volume_all: CiftiReplaceStructureVolumeAllParamsDict | None = None,
    label: list[CiftiReplaceStructureLabelParamsDict] | None = None,
    metric: list[CiftiReplaceStructureMetricParamsDict] | None = None,
    volume: list[CiftiReplaceStructureVolumeParamsDict] | None = None,
    action: str | None = None,
    discard_unused_labels: bool = False,
    runner: Runner | None = None,
) -> CiftiReplaceStructureOutputs:
    """
    REPLACE DATA IN A STRUCTURE IN A CIFTI FILE.
    
    This is a fairly low-level command, you probably want to use
    -cifti-create-dense-from-template instead.
    
    You must specify at least one of -metric, -label, -volume, or -volume-all
    for this command to do anything. Input volumes must line up with the output
    of -cifti-separate. For dtseries/dscalar, use COLUMN, and if your dconn
    matrix will be fully symmetric, COLUMN is more efficient. The -volume-all
    option must not be specified when using a -volume option. A -metric option
    must not be specified when using a -label option, and is not recommended on
    a label-type cifti file. For each <structure> argument, use one of the
    following strings:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        cifti: the cifti to modify.
        direction: which dimension to interpret as a single map, ROW or COLUMN.
        volume_all: replace the data in all volume components.
        label: replace the data in a surface label component.
        metric: replace the data in a surface component.
        volume: replace the data in a volume component.
        action: how to handle conflicts between label keys\
            \
            'ERROR', 'LEFT_SURFACE_FIRST', or 'LEGACY', default 'ERROR', use\
            'LEGACY' to match v1.4.2 and earlier.
        discard_unused_labels: when operating on a dlabel file, drop any unused\
            label keys from the label table.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiReplaceStructureOutputs`).
    """
    params = cifti_replace_structure_params(
        cifti=cifti,
        direction=direction,
        volume_all=volume_all,
        label=label,
        metric=metric,
        volume=volume,
        action=action,
        discard_unused_labels=discard_unused_labels,
    )
    return cifti_replace_structure_execute(params, runner)


__all__ = [
    "CIFTI_REPLACE_STRUCTURE_METADATA",
    "CiftiReplaceStructureLabelParamsDict",
    "CiftiReplaceStructureLabelParamsDictTagged",
    "CiftiReplaceStructureMetricParamsDict",
    "CiftiReplaceStructureMetricParamsDictTagged",
    "CiftiReplaceStructureOutputs",
    "CiftiReplaceStructureParamsDict",
    "CiftiReplaceStructureParamsDictTagged",
    "CiftiReplaceStructureVolumeAllParamsDict",
    "CiftiReplaceStructureVolumeAllParamsDictTagged",
    "CiftiReplaceStructureVolumeParamsDict",
    "CiftiReplaceStructureVolumeParamsDictTagged",
    "cifti_replace_structure",
    "cifti_replace_structure_execute",
    "cifti_replace_structure_label",
    "cifti_replace_structure_metric",
    "cifti_replace_structure_params",
    "cifti_replace_structure_volume",
    "cifti_replace_structure_volume_all",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_ERODE_METADATA = Metadata(
    id="de54c7f54c6368999f264ce5975a56da07dfce18.workbench",
    name="volume-erode",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeErodeParamsDictNoTag = typing.TypedDict('_VolumeErodeParamsDictNoTag', {
    "volume": InputPathType,
    "distance": float,
    "volume-out": str,
    "subvol": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
})
VolumeErodeParamsDictTagged = typing.TypedDict('VolumeErodeParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-erode"],
    "volume": InputPathType,
    "distance": float,
    "volume-out": str,
    "subvol": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
})
VolumeErodeParamsDict = _VolumeErodeParamsDictNoTag | VolumeErodeParamsDictTagged


class VolumeErodeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeErodeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_erode_params(
    volume: InputPathType,
    distance: float,
    volume_out: str,
    subvol: str | None = None,
    roi_volume: InputPathType | None = None,
) -> VolumeErodeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume: the volume to erode.
        distance: distance in mm to erode.
        volume_out: the output volume.
        subvol: select a single subvolume to dilate\
            \
            the subvolume number or name.
        roi_volume: assume voxels outside this roi are nonzero\
            \
            volume file, positive values denote voxels that have data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-erode",
        "volume": volume,
        "distance": distance,
        "volume-out": volume_out,
    }
    if subvol is not None:
        params["subvol"] = subvol
    if roi_volume is not None:
        params["roi-volume"] = roi_volume
    return params


def volume_erode_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeErodeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume", None) is None:
        raise StyxValidationError("`volume` must not be None")
    if not isinstance(params["volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `InputPathType`')
    if params.get("distance", None) is None:
        raise StyxValidationError("`distance` must not be None")
    if not isinstance(params["distance"], (float, int)):
        raise StyxValidationError(f'`distance` has the wrong type: Received `{type(params.get("distance", None))}` expected `float`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("subvol", None) is not None:
        if not isinstance(params["subvol"], str):
            raise StyxValidationError(f'`subvol` has the wrong type: Received `{type(params.get("subvol", None))}` expected `str | None`')
    if params.get("roi-volume", None) is not None:
        if not isinstance(params["roi-volume"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-volume` has the wrong type: Received `{type(params.get("roi-volume", None))}` expected `InputPathType | None`')


def volume_erode_cargs(
    params: VolumeErodeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-erode"
    ])
    cargs.append(execution.input_file(params.get("volume", None)))
    cargs.append(str(params.get("distance", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvol", None)
        ])
    if params.get("roi-volume", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-volume", None))
        ])
    return cargs


def volume_erode_outputs(
    params: VolumeErodeParamsDict,
    execution: Execution,
) -> VolumeErodeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeErodeOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_erode_execute(
    params: VolumeErodeParamsDict,
    runner: Runner | None = None,
) -> VolumeErodeOutputs:
    """
    ERODE A VOLUME FILE.
    
    Around each voxel with a value of zero, set surrounding voxels to zero. The
    surrounding voxels are all face neighbors and all voxels within the
    specified distance (center to center).
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeErodeOutputs`).
    """
    volume_erode_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_ERODE_METADATA)
    params = execution.params(params)
    cargs = volume_erode_cargs(params, execution)
    ret = volume_erode_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_erode(
    volume: InputPathType,
    distance: float,
    volume_out: str,
    subvol: str | None = None,
    roi_volume: InputPathType | None = None,
    runner: Runner | None = None,
) -> VolumeErodeOutputs:
    """
    ERODE A VOLUME FILE.
    
    Around each voxel with a value of zero, set surrounding voxels to zero. The
    surrounding voxels are all face neighbors and all voxels within the
    specified distance (center to center).
    
    Args:
        volume: the volume to erode.
        distance: distance in mm to erode.
        volume_out: the output volume.
        subvol: select a single subvolume to dilate\
            \
            the subvolume number or name.
        roi_volume: assume voxels outside this roi are nonzero\
            \
            volume file, positive values denote voxels that have data.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeErodeOutputs`).
    """
    params = volume_erode_params(
        volume=volume,
        distance=distance,
        volume_out=volume_out,
        subvol=subvol,
        roi_volume=roi_volume,
    )
    return volume_erode_execute(params, runner)


__all__ = [
    "VOLUME_ERODE_METADATA",
    "VolumeErodeOutputs",
    "VolumeErodeParamsDict",
    "VolumeErodeParamsDictTagged",
    "volume_erode",
    "volume_erode_execute",
    "volume_erode_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_CREATE_METADATA = Metadata(
    id="0ea13854b7a75326c4a105c15f5a5b8318a5d533.workbench",
    name="volume-create",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeCreatePlumbParamsDictNoTag = typing.TypedDict('_VolumeCreatePlumbParamsDictNoTag', {
    "axis-order": str,
    "x-spacing": float,
    "y-spacing": float,
    "z-spacing": float,
    "x-offset": float,
    "y-offset": float,
    "z-offset": float,
})
VolumeCreatePlumbParamsDictTagged = typing.TypedDict('VolumeCreatePlumbParamsDictTagged', {
    "@type": typing.Literal["plumb"],
    "axis-order": str,
    "x-spacing": float,
    "y-spacing": float,
    "z-spacing": float,
    "x-offset": float,
    "y-offset": float,
    "z-offset": float,
})
VolumeCreatePlumbParamsDict = _VolumeCreatePlumbParamsDictNoTag | VolumeCreatePlumbParamsDictTagged


_VolumeCreateSformParamsDictNoTag = typing.TypedDict('_VolumeCreateSformParamsDictNoTag', {
    "xi-spacing": float,
    "xj-spacing": float,
    "xk-spacing": float,
    "x-offset": float,
    "yi-spacing": float,
    "yj-spacing": float,
    "yk-spacing": float,
    "y-offset": float,
    "zi-spacing": float,
    "zj-spacing": float,
    "zk-spacing": float,
    "z-offset": float,
})
VolumeCreateSformParamsDictTagged = typing.TypedDict('VolumeCreateSformParamsDictTagged', {
    "@type": typing.Literal["sform"],
    "xi-spacing": float,
    "xj-spacing": float,
    "xk-spacing": float,
    "x-offset": float,
    "yi-spacing": float,
    "yj-spacing": float,
    "yk-spacing": float,
    "y-offset": float,
    "zi-spacing": float,
    "zj-spacing": float,
    "zk-spacing": float,
    "z-offset": float,
})
VolumeCreateSformParamsDict = _VolumeCreateSformParamsDictNoTag | VolumeCreateSformParamsDictTagged


_VolumeCreateParamsDictNoTag = typing.TypedDict('_VolumeCreateParamsDictNoTag', {
    "i-dim": int,
    "j-dim": int,
    "k-dim": int,
    "volume-out": str,
    "plumb": typing.NotRequired[VolumeCreatePlumbParamsDict | None],
    "sform": typing.NotRequired[VolumeCreateSformParamsDict | None],
})
VolumeCreateParamsDictTagged = typing.TypedDict('VolumeCreateParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-create"],
    "i-dim": int,
    "j-dim": int,
    "k-dim": int,
    "volume-out": str,
    "plumb": typing.NotRequired[VolumeCreatePlumbParamsDict | None],
    "sform": typing.NotRequired[VolumeCreateSformParamsDict | None],
})
VolumeCreateParamsDict = _VolumeCreateParamsDictNoTag | VolumeCreateParamsDictTagged


def volume_create_plumb(
    axis_order: str,
    x_spacing: float,
    y_spacing: float,
    z_spacing: float,
    x_offset: float,
    y_offset: float,
    z_offset: float,
) -> VolumeCreatePlumbParamsDictTagged:
    """
    Build parameters.
    
    Args:
        axis_order: a string like 'XYZ' that specifies which index is along\
            which spatial dimension.
        x_spacing: change in x-coordinate from incrementing the relevant index.
        y_spacing: change in y-coordinate from incrementing the relevant index.
        z_spacing: change in z-coordinate from incrementing the relevant index.
        x_offset: the x-coordinate of the center of the first voxel.
        y_offset: the y-coordinate of the center of the first voxel.
        z_offset: the z-coordinate of the center of the first voxel.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "plumb",
        "axis-order": axis_order,
        "x-spacing": x_spacing,
        "y-spacing": y_spacing,
        "z-spacing": z_spacing,
        "x-offset": x_offset,
        "y-offset": y_offset,
        "z-offset": z_offset,
    }
    return params


def volume_create_plumb_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeCreatePlumbParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("axis-order", None) is None:
        raise StyxValidationError("`axis-order` must not be None")
    if not isinstance(params["axis-order"], str):
        raise StyxValidationError(f'`axis-order` has the wrong type: Received `{type(params.get("axis-order", None))}` expected `str`')
    if params.get("x-spacing", None) is None:
        raise StyxValidationError("`x-spacing` must not be None")
    if not isinstance(params["x-spacing"], (float, int)):
        raise StyxValidationError(f'`x-spacing` has the wrong type: Received `{type(params.get("x-spacing", None))}` expected `float`')
    if params.get("y-spacing", None) is None:
        raise StyxValidationError("`y-spacing` must not be None")
    if not isinstance(params["y-spacing"], (float, int)):
        raise StyxValidationError(f'`y-spacing` has the wrong type: Received `{type(params.get("y-spacing", None))}` expected `float`')
    if params.get("z-spacing", None) is None:
        raise StyxValidationError("`z-spacing` must not be None")
    if not isinstance(params["z-spacing"], (float, int)):
        raise StyxValidationError(f'`z-spacing` has the wrong type: Received `{type(params.get("z-spacing", None))}` expected `float`')
    if params.get("x-offset", None) is None:
        raise StyxValidationError("`x-offset` must not be None")
    if not isinstance(params["x-offset"], (float, int)):
        raise StyxValidationError(f'`x-offset` has the wrong type: Received `{type(params.get("x-offset", None))}` expected `float`')
    if params.get("y-offset", None) is None:
        raise StyxValidationError("`y-offset` must not be None")
    if not isinstance(params["y-offset"], (float, int)):
        raise StyxValidationError(f'`y-offset` has the wrong type: Received `{type(params.get("y-offset", None))}` expected `float`')
    if params.get("z-offset", None) is None:
        raise StyxValidationError("`z-offset` must not be None")
    if not isinstance(params["z-offset"], (float, int)):
        raise StyxValidationError(f'`z-offset` has the wrong type: Received `{type(params.get("z-offset", None))}` expected `float`')


def volume_create_plumb_cargs(
    params: VolumeCreatePlumbParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-plumb",
        params.get("axis-order", None),
        str(params.get("x-spacing", None)),
        str(params.get("y-spacing", None)),
        str(params.get("z-spacing", None)),
        str(params.get("x-offset", None)),
        str(params.get("y-offset", None)),
        str(params.get("z-offset", None))
    ])
    return cargs


def volume_create_sform(
    xi_spacing: float,
    xj_spacing: float,
    xk_spacing: float,
    x_offset: float,
    yi_spacing: float,
    yj_spacing: float,
    yk_spacing: float,
    y_offset: float,
    zi_spacing: float,
    zj_spacing: float,
    zk_spacing: float,
    z_offset: float,
) -> VolumeCreateSformParamsDictTagged:
    """
    Build parameters.
    
    Args:
        xi_spacing: increase in x coordinate from incrementing the i index.
        xj_spacing: increase in x coordinate from incrementing the j index.
        xk_spacing: increase in x coordinate from incrementing the k index.
        x_offset: x coordinate of first voxel.
        yi_spacing: increase in y coordinate from incrementing the i index.
        yj_spacing: increase in y coordinate from incrementing the j index.
        yk_spacing: increase in y coordinate from incrementing the k index.
        y_offset: y coordinate of first voxel.
        zi_spacing: increase in z coordinate from incrementing the i index.
        zj_spacing: increase in z coordinate from incrementing the j index.
        zk_spacing: increase in z coordinate from incrementing the k index.
        z_offset: z coordinate of first voxel.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "sform",
        "xi-spacing": xi_spacing,
        "xj-spacing": xj_spacing,
        "xk-spacing": xk_spacing,
        "x-offset": x_offset,
        "yi-spacing": yi_spacing,
        "yj-spacing": yj_spacing,
        "yk-spacing": yk_spacing,
        "y-offset": y_offset,
        "zi-spacing": zi_spacing,
        "zj-spacing": zj_spacing,
        "zk-spacing": zk_spacing,
        "z-offset": z_offset,
    }
    return params


def volume_create_sform_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeCreateSformParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("xi-spacing", None) is None:
        raise StyxValidationError("`xi-spacing` must not be None")
    if not isinstance(params["xi-spacing"], (float, int)):
        raise StyxValidationError(f'`xi-spacing` has the wrong type: Received `{type(params.get("xi-spacing", None))}` expected `float`')
    if params.get("xj-spacing", None) is None:
        raise StyxValidationError("`xj-spacing` must not be None")
    if not isinstance(params["xj-spacing"], (float, int)):
        raise StyxValidationError(f'`xj-spacing` has the wrong type: Received `{type(params.get("xj-spacing", None))}` expected `float`')
    if params.get("xk-spacing", None) is None:
        raise StyxValidationError("`xk-spacing` must not be None")
    if not isinstance(params["xk-spacing"], (float, int)):
        raise StyxValidationError(f'`xk-spacing` has the wrong type: Received `{type(params.get("xk-spacing", None))}` expected `float`')
    if params.get("x-offset", None) is None:
        raise StyxValidationError("`x-offset` must not be None")
    if not isinstance(params["x-offset"], (float, int)):
        raise StyxValidationError(f'`x-offset` has the wrong type: Received `{type(params.get("x-offset", None))}` expected `float`')
    if params.get("yi-spacing", None) is None:
        raise StyxValidationError("`yi-spacing` must not be None")
    if not isinstance(params["yi-spacing"], (float, int)):
        raise StyxValidationError(f'`yi-spacing` has the wrong type: Received `{type(params.get("yi-spacing", None))}` expected `float`')
    if params.get("yj-spacing", None) is None:
        raise StyxValidationError("`yj-spacing` must not be None")
    if not isinstance(params["yj-spacing"], (float, int)):
        raise StyxValidationError(f'`yj-spacing` has the wrong type: Received `{type(params.get("yj-spacing", None))}` expected `float`')
    if params.get("yk-spacing", None) is None:
        raise StyxValidationError("`yk-spacing` must not be None")
    if not isinstance(params["yk-spacing"], (float, int)):
        raise StyxValidationError(f'`yk-spacing` has the wrong type: Received `{type(params.get("yk-spacing", None))}` expected `float`')
    if params.get("y-offset", None) is None:
        raise StyxValidationError("`y-offset` must not be None")
    if not isinstance(params["y-offset"], (float, int)):
        raise StyxValidationError(f'`y-offset` has the wrong type: Received `{type(params.get("y-offset", None))}` expected `float`')
    if params.get("zi-spacing", None) is None:
        raise StyxValidationError("`zi-spacing` must not be None")
    if not isinstance(params["zi-spacing"], (float, int)):
        raise StyxValidationError(f'`zi-spacing` has the wrong type: Received `{type(params.get("zi-spacing", None))}` expected `float`')
    if params.get("zj-spacing", None) is None:
        raise StyxValidationError("`zj-spacing` must not be None")
    if not isinstance(params["zj-spacing"], (float, int)):
        raise StyxValidationError(f'`zj-spacing` has the wrong type: Received `{type(params.get("zj-spacing", None))}` expected `float`')
    if params.get("zk-spacing", None) is None:
        raise StyxValidationError("`zk-spacing` must not be None")
    if not isinstance(params["zk-spacing"], (float, int)):
        raise StyxValidationError(f'`zk-spacing` has the wrong type: Received `{type(params.get("zk-spacing", None))}` expected `float`')
    if params.get("z-offset", None) is None:
        raise StyxValidationError("`z-offset` must not be None")
    if not isinstance(params["z-offset"], (float, int)):
        raise StyxValidationError(f'`z-offset` has the wrong type: Received `{type(params.get("z-offset", None))}` expected `float`')


def volume_create_sform_cargs(
    params: VolumeCreateSformParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-sform",
        str(params.get("xi-spacing", None)),
        str(params.get("xj-spacing", None)),
        str(params.get("xk-spacing", None)),
        str(params.get("x-offset", None)),
        str(params.get("yi-spacing", None)),
        str(params.get("yj-spacing", None)),
        str(params.get("yk-spacing", None)),
        str(params.get("y-offset", None)),
        str(params.get("zi-spacing", None)),
        str(params.get("zj-spacing", None)),
        str(params.get("zk-spacing", None)),
        str(params.get("z-offset", None))
    ])
    return cargs


class VolumeCreateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeCreateParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_create_params(
    i_dim: int,
    j_dim: int,
    k_dim: int,
    volume_out: str,
    plumb: VolumeCreatePlumbParamsDict | None = None,
    sform: VolumeCreateSformParamsDict | None = None,
) -> VolumeCreateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        i_dim: length of first dimension.
        j_dim: length of second dimension.
        k_dim: length of third dimension.
        volume_out: the output volume.
        plumb: set via axis order and spacing/offset.
        sform: set via a nifti sform.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-create",
        "i-dim": i_dim,
        "j-dim": j_dim,
        "k-dim": k_dim,
        "volume-out": volume_out,
    }
    if plumb is not None:
        params["plumb"] = plumb
    if sform is not None:
        params["sform"] = sform
    return params


def volume_create_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeCreateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("i-dim", None) is None:
        raise StyxValidationError("`i-dim` must not be None")
    if not isinstance(params["i-dim"], int):
        raise StyxValidationError(f'`i-dim` has the wrong type: Received `{type(params.get("i-dim", None))}` expected `int`')
    if params.get("j-dim", None) is None:
        raise StyxValidationError("`j-dim` must not be None")
    if not isinstance(params["j-dim"], int):
        raise StyxValidationError(f'`j-dim` has the wrong type: Received `{type(params.get("j-dim", None))}` expected `int`')
    if params.get("k-dim", None) is None:
        raise StyxValidationError("`k-dim` must not be None")
    if not isinstance(params["k-dim"], int):
        raise StyxValidationError(f'`k-dim` has the wrong type: Received `{type(params.get("k-dim", None))}` expected `int`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("plumb", None) is not None:
        volume_create_plumb_validate(params["plumb"])
    if params.get("sform", None) is not None:
        volume_create_sform_validate(params["sform"])


def volume_create_cargs(
    params: VolumeCreateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-create"
    ])
    cargs.append(str(params.get("i-dim", None)))
    cargs.append(str(params.get("j-dim", None)))
    cargs.append(str(params.get("k-dim", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("plumb", None) is not None or params.get("sform", None) is not None:
        cargs.extend([
            *(volume_create_plumb_cargs(params.get("plumb", None), execution) if (params.get("plumb", None) is not None) else []),
            *(volume_create_sform_cargs(params.get("sform", None), execution) if (params.get("sform", None) is not None) else [])
        ])
    return cargs


def volume_create_outputs(
    params: VolumeCreateParamsDict,
    execution: Execution,
) -> VolumeCreateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeCreateOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_create_execute(
    params: VolumeCreateParamsDict,
    runner: Runner | None = None,
) -> VolumeCreateOutputs:
    """
    CREATE A BLANK VOLUME FILE.
    
    Creates a volume file full of zeros. Exactly one of -plumb or -sform must be
    specified.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeCreateOutputs`).
    """
    volume_create_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_CREATE_METADATA)
    params = execution.params(params)
    cargs = volume_create_cargs(params, execution)
    ret = volume_create_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_create(
    i_dim: int,
    j_dim: int,
    k_dim: int,
    volume_out: str,
    plumb: VolumeCreatePlumbParamsDict | None = None,
    sform: VolumeCreateSformParamsDict | None = None,
    runner: Runner | None = None,
) -> VolumeCreateOutputs:
    """
    CREATE A BLANK VOLUME FILE.
    
    Creates a volume file full of zeros. Exactly one of -plumb or -sform must be
    specified.
    
    Args:
        i_dim: length of first dimension.
        j_dim: length of second dimension.
        k_dim: length of third dimension.
        volume_out: the output volume.
        plumb: set via axis order and spacing/offset.
        sform: set via a nifti sform.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeCreateOutputs`).
    """
    params = volume_create_params(
        i_dim=i_dim,
        j_dim=j_dim,
        k_dim=k_dim,
        volume_out=volume_out,
        plumb=plumb,
        sform=sform,
    )
    return volume_create_execute(params, runner)


__all__ = [
    "VOLUME_CREATE_METADATA",
    "VolumeCreateOutputs",
    "VolumeCreateParamsDict",
    "VolumeCreateParamsDictTagged",
    "VolumeCreatePlumbParamsDict",
    "VolumeCreatePlumbParamsDictTagged",
    "VolumeCreateSformParamsDict",
    "VolumeCreateSformParamsDictTagged",
    "volume_create",
    "volume_create_execute",
    "volume_create_params",
    "volume_create_plumb",
    "volume_create_sform",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL_MASK_METADATA = Metadata(
    id="8143b670e00529bec92d2236053b58d9bab6aa76.workbench",
    name="label-mask",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_LabelMaskParamsDictNoTag = typing.TypedDict('_LabelMaskParamsDictNoTag', {
    "label": InputPathType,
    "mask": InputPathType,
    "label-out": str,
    "column": typing.NotRequired[str | None],
})
LabelMaskParamsDictTagged = typing.TypedDict('LabelMaskParamsDictTagged', {
    "@type": typing.Literal["workbench/label-mask"],
    "label": InputPathType,
    "mask": InputPathType,
    "label-out": str,
    "column": typing.NotRequired[str | None],
})
LabelMaskParamsDict = _LabelMaskParamsDictNoTag | LabelMaskParamsDictTagged


class LabelMaskOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelMaskParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_out: OutputPathType
    """the output label file"""


def label_mask_params(
    label: InputPathType,
    mask: InputPathType,
    label_out: str,
    column: str | None = None,
) -> LabelMaskParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label: the label file to mask.
        mask: the mask metric.
        label_out: the output label file.
        column: select a single column\
            \
            the column number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/label-mask",
        "label": label,
        "mask": mask,
        "label-out": label_out,
    }
    if column is not None:
        params["column"] = column
    return params


def label_mask_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelMaskParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label", None) is None:
        raise StyxValidationError("`label` must not be None")
    if not isinstance(params["label"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `InputPathType`')
    if params.get("mask", None) is None:
        raise StyxValidationError("`mask` must not be None")
    if not isinstance(params["mask"], (pathlib.Path, str)):
        raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType`')
    if params.get("label-out", None) is None:
        raise StyxValidationError("`label-out` must not be None")
    if not isinstance(params["label-out"], str):
        raise StyxValidationError(f'`label-out` has the wrong type: Received `{type(params.get("label-out", None))}` expected `str`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')


def label_mask_cargs(
    params: LabelMaskParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-label-mask"
    ])
    cargs.append(execution.input_file(params.get("label", None)))
    cargs.append(execution.input_file(params.get("mask", None)))
    cargs.append(params.get("label-out", None))
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    return cargs


def label_mask_outputs(
    params: LabelMaskParamsDict,
    execution: Execution,
) -> LabelMaskOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelMaskOutputs(
        root=execution.output_file("."),
        label_out=execution.output_file(params.get("label-out", None)),
    )
    return ret


def label_mask_execute(
    params: LabelMaskParamsDict,
    runner: Runner | None = None,
) -> LabelMaskOutputs:
    """
    MASK A LABEL FILE.
    
    By default, the output label is a copy of the input label, but with the
    'unused' label wherever the mask metric is zero or negative. if -column is
    specified, the output contains only one column, the masked version of the
    specified input column.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelMaskOutputs`).
    """
    label_mask_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_MASK_METADATA)
    params = execution.params(params)
    cargs = label_mask_cargs(params, execution)
    ret = label_mask_outputs(params, execution)
    execution.run(cargs)
    return ret


def label_mask(
    label: InputPathType,
    mask: InputPathType,
    label_out: str,
    column: str | None = None,
    runner: Runner | None = None,
) -> LabelMaskOutputs:
    """
    MASK A LABEL FILE.
    
    By default, the output label is a copy of the input label, but with the
    'unused' label wherever the mask metric is zero or negative. if -column is
    specified, the output contains only one column, the masked version of the
    specified input column.
    
    Args:
        label: the label file to mask.
        mask: the mask metric.
        label_out: the output label file.
        column: select a single column\
            \
            the column number or name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelMaskOutputs`).
    """
    params = label_mask_params(
        label=label,
        mask=mask,
        label_out=label_out,
        column=column,
    )
    return label_mask_execute(params, runner)


__all__ = [
    "LABEL_MASK_METADATA",
    "LabelMaskOutputs",
    "LabelMaskParamsDict",
    "LabelMaskParamsDictTagged",
    "label_mask",
    "label_mask_execute",
    "label_mask_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_CORRELATION_METADATA = Metadata(
    id="dcfca8ed7e2b49948aa64aa87d5693dc899aee50.workbench",
    name="cifti-correlation",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiCorrelationRoiOverrideParamsDictNoTag = typing.TypedDict('_CiftiCorrelationRoiOverrideParamsDictNoTag', {
    "roi-cifti": typing.NotRequired[InputPathType | None],
    "roi-vol": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationRoiOverrideParamsDictTagged = typing.TypedDict('CiftiCorrelationRoiOverrideParamsDictTagged', {
    "@type": typing.Literal["roi-override"],
    "roi-cifti": typing.NotRequired[InputPathType | None],
    "roi-vol": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationRoiOverrideParamsDict = _CiftiCorrelationRoiOverrideParamsDictNoTag | CiftiCorrelationRoiOverrideParamsDictTagged


_CiftiCorrelationParamsDictNoTag = typing.TypedDict('_CiftiCorrelationParamsDictNoTag', {
    "cifti": InputPathType,
    "cifti-out": str,
    "roi-override": typing.NotRequired[CiftiCorrelationRoiOverrideParamsDict | None],
    "limit-GB": typing.NotRequired[float | None],
    "weight-file": typing.NotRequired[str | None],
    "covariance": bool,
    "no-demean": bool,
    "fisher-z": bool,
})
CiftiCorrelationParamsDictTagged = typing.TypedDict('CiftiCorrelationParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-correlation"],
    "cifti": InputPathType,
    "cifti-out": str,
    "roi-override": typing.NotRequired[CiftiCorrelationRoiOverrideParamsDict | None],
    "limit-GB": typing.NotRequired[float | None],
    "weight-file": typing.NotRequired[str | None],
    "covariance": bool,
    "no-demean": bool,
    "fisher-z": bool,
})
CiftiCorrelationParamsDict = _CiftiCorrelationParamsDictNoTag | CiftiCorrelationParamsDictTagged


def cifti_correlation_roi_override(
    roi_cifti: InputPathType | None = None,
    roi_vol: InputPathType | None = None,
    roi_metric: InputPathType | None = None,
    roi_metric_: InputPathType | None = None,
    roi_metric_2: InputPathType | None = None,
) -> CiftiCorrelationRoiOverrideParamsDictTagged:
    """
    Build parameters.
    
    Args:
        roi_cifti: use a cifti file for combined rois\
            \
            the cifti roi file.
        roi_vol: use an roi for volume\
            \
            the volume roi file.
        roi_metric: use an roi for cerebellum\
            \
            the cerebellum roi as a metric file.
        roi_metric_: use an roi for right hempsphere\
            \
            the right roi as a metric file.
        roi_metric_2: use an roi for left hempsphere\
            \
            the left roi as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi-override",
    }
    if roi_cifti is not None:
        params["roi-cifti"] = roi_cifti
    if roi_vol is not None:
        params["roi-vol"] = roi_vol
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    if roi_metric_ is not None:
        params["roi-metric"] = roi_metric_
    if roi_metric_2 is not None:
        params["roi-metric"] = roi_metric_2
    return params


def cifti_correlation_roi_override_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCorrelationRoiOverrideParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-cifti", None) is not None:
        if not isinstance(params["roi-cifti"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-cifti` has the wrong type: Received `{type(params.get("roi-cifti", None))}` expected `InputPathType | None`')
    if params.get("roi-vol", None) is not None:
        if not isinstance(params["roi-vol"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-vol` has the wrong type: Received `{type(params.get("roi-vol", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def cifti_correlation_roi_override_cargs(
    params: CiftiCorrelationRoiOverrideParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-roi-override")
    if params.get("roi-cifti", None) is not None:
        cargs.extend([
            "-cifti-roi",
            execution.input_file(params.get("roi-cifti", None))
        ])
    if params.get("roi-vol", None) is not None:
        cargs.extend([
            "-vol-roi",
            execution.input_file(params.get("roi-vol", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-cerebellum-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-right-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-left-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


class CiftiCorrelationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiCorrelationParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""


def cifti_correlation_params(
    cifti: InputPathType,
    cifti_out: str,
    roi_override: CiftiCorrelationRoiOverrideParamsDict | None = None,
    limit_gb: float | None = None,
    weight_file: str | None = None,
    covariance: bool = False,
    no_demean: bool = False,
    fisher_z: bool = False,
) -> CiftiCorrelationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti: input cifti file.
        cifti_out: output cifti file.
        roi_override: perform correlation from a subset of rows to all rows.
        limit_gb: restrict memory usage\
            \
            memory limit in gigabytes.
        weight_file: specify column weights\
            \
            text file containing one weight per column.
        covariance: compute covariance instead of correlation.
        no_demean: instead of correlation, do dot product of rows, then\
            normalize by diagonal.
        fisher_z: apply fisher small z transform (ie, artanh) to correlation.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-correlation",
        "cifti": cifti,
        "cifti-out": cifti_out,
        "covariance": covariance,
        "no-demean": no_demean,
        "fisher-z": fisher_z,
    }
    if roi_override is not None:
        params["roi-override"] = roi_override
    if limit_gb is not None:
        params["limit-GB"] = limit_gb
    if weight_file is not None:
        params["weight-file"] = weight_file
    return params


def cifti_correlation_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCorrelationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti", None) is None:
        raise StyxValidationError("`cifti` must not be None")
    if not isinstance(params["cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `InputPathType`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("roi-override", None) is not None:
        cifti_correlation_roi_override_validate(params["roi-override"])
    if params.get("limit-GB", None) is not None:
        if not isinstance(params["limit-GB"], (float, int)):
            raise StyxValidationError(f'`limit-GB` has the wrong type: Received `{type(params.get("limit-GB", None))}` expected `float | None`')
    if params.get("weight-file", None) is not None:
        if not isinstance(params["weight-file"], str):
            raise StyxValidationError(f'`weight-file` has the wrong type: Received `{type(params.get("weight-file", None))}` expected `str | None`')
    if params.get("covariance", False) is None:
        raise StyxValidationError("`covariance` must not be None")
    if not isinstance(params["covariance"], bool):
        raise StyxValidationError(f'`covariance` has the wrong type: Received `{type(params.get("covariance", False))}` expected `bool`')
    if params.get("no-demean", False) is None:
        raise StyxValidationError("`no-demean` must not be None")
    if not isinstance(params["no-demean"], bool):
        raise StyxValidationError(f'`no-demean` has the wrong type: Received `{type(params.get("no-demean", False))}` expected `bool`')
    if params.get("fisher-z", False) is None:
        raise StyxValidationError("`fisher-z` must not be None")
    if not isinstance(params["fisher-z"], bool):
        raise StyxValidationError(f'`fisher-z` has the wrong type: Received `{type(params.get("fisher-z", False))}` expected `bool`')


def cifti_correlation_cargs(
    params: CiftiCorrelationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-correlation"
    ])
    cargs.append(execution.input_file(params.get("cifti", None)))
    cargs.append(params.get("cifti-out", None))
    if params.get("roi-override", None) is not None:
        cargs.extend(cifti_correlation_roi_override_cargs(params.get("roi-override", None), execution))
    if params.get("limit-GB", None) is not None:
        cargs.extend([
            "-mem-limit",
            str(params.get("limit-GB", None))
        ])
    if params.get("weight-file", None) is not None:
        cargs.extend([
            "-weights",
            params.get("weight-file", None)
        ])
    if params.get("covariance", False):
        cargs.append("-covariance")
    if params.get("no-demean", False):
        cargs.append("-no-demean")
    if params.get("fisher-z", False):
        cargs.append("-fisher-z")
    return cargs


def cifti_correlation_outputs(
    params: CiftiCorrelationParamsDict,
    execution: Execution,
) -> CiftiCorrelationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiCorrelationOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_correlation_execute(
    params: CiftiCorrelationParamsDict,
    runner: Runner | None = None,
) -> CiftiCorrelationOutputs:
    """
    GENERATE CORRELATION OF ROWS IN A CIFTI FILE.
    
    For each row (or each row inside an roi if -roi-override is specified),
    correlate to all other rows. The -cifti-roi suboption to -roi-override may
    not be specified with any other -*-roi suboption, but you may specify the
    other -*-roi suboptions together.
    
    When using the -fisher-z option, the output is NOT a Z-score, it is
    artanh(r), to do further math on this output, consider using -cifti-math.
    
    Restricting the memory usage will make it calculate the output in chunks,
    and if the input file size is more than 70% of the memory limit, it will
    also read through the input file as rows are required, resulting in several
    passes through the input file (once per chunk). Memory limit does not need
    to be an integer, you may also specify 0 to calculate a single output row at
    a time (this may be very slow).
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCorrelationOutputs`).
    """
    cifti_correlation_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CORRELATION_METADATA)
    params = execution.params(params)
    cargs = cifti_correlation_cargs(params, execution)
    ret = cifti_correlation_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_correlation(
    cifti: InputPathType,
    cifti_out: str,
    roi_override: CiftiCorrelationRoiOverrideParamsDict | None = None,
    limit_gb: float | None = None,
    weight_file: str | None = None,
    covariance: bool = False,
    no_demean: bool = False,
    fisher_z: bool = False,
    runner: Runner | None = None,
) -> CiftiCorrelationOutputs:
    """
    GENERATE CORRELATION OF ROWS IN A CIFTI FILE.
    
    For each row (or each row inside an roi if -roi-override is specified),
    correlate to all other rows. The -cifti-roi suboption to -roi-override may
    not be specified with any other -*-roi suboption, but you may specify the
    other -*-roi suboptions together.
    
    When using the -fisher-z option, the output is NOT a Z-score, it is
    artanh(r), to do further math on this output, consider using -cifti-math.
    
    Restricting the memory usage will make it calculate the output in chunks,
    and if the input file size is more than 70% of the memory limit, it will
    also read through the input file as rows are required, resulting in several
    passes through the input file (once per chunk). Memory limit does not need
    to be an integer, you may also specify 0 to calculate a single output row at
    a time (this may be very slow).
    
    Args:
        cifti: input cifti file.
        cifti_out: output cifti file.
        roi_override: perform correlation from a subset of rows to all rows.
        limit_gb: restrict memory usage\
            \
            memory limit in gigabytes.
        weight_file: specify column weights\
            \
            text file containing one weight per column.
        covariance: compute covariance instead of correlation.
        no_demean: instead of correlation, do dot product of rows, then\
            normalize by diagonal.
        fisher_z: apply fisher small z transform (ie, artanh) to correlation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCorrelationOutputs`).
    """
    params = cifti_correlation_params(
        cifti=cifti,
        cifti_out=cifti_out,
        roi_override=roi_override,
        limit_gb=limit_gb,
        weight_file=weight_file,
        covariance=covariance,
        no_demean=no_demean,
        fisher_z=fisher_z,
    )
    return cifti_correlation_execute(params, runner)


__all__ = [
    "CIFTI_CORRELATION_METADATA",
    "CiftiCorrelationOutputs",
    "CiftiCorrelationParamsDict",
    "CiftiCorrelationParamsDictTagged",
    "CiftiCorrelationRoiOverrideParamsDict",
    "CiftiCorrelationRoiOverrideParamsDictTagged",
    "cifti_correlation",
    "cifti_correlation_execute",
    "cifti_correlation_params",
    "cifti_correlation_roi_override",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_COMPONENTS_TO_FRAMES_METADATA = Metadata(
    id="62835d3af56370841cd14095a4b69710f24db721.workbench",
    name="volume-components-to-frames",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeComponentsToFramesParamsDictNoTag = typing.TypedDict('_VolumeComponentsToFramesParamsDictNoTag', {
    "input": InputPathType,
    "output": str,
})
VolumeComponentsToFramesParamsDictTagged = typing.TypedDict('VolumeComponentsToFramesParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-components-to-frames"],
    "input": InputPathType,
    "output": str,
})
VolumeComponentsToFramesParamsDict = _VolumeComponentsToFramesParamsDictNoTag | VolumeComponentsToFramesParamsDictTagged


class VolumeComponentsToFramesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeComponentsToFramesParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the input volume converted to multiple frames of scalar type"""


def volume_components_to_frames_params(
    input_: InputPathType,
    output: str,
) -> VolumeComponentsToFramesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the RGB/complex-type volume.
        output: the input volume converted to multiple frames of scalar type.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-components-to-frames",
        "input": input_,
        "output": output,
    }
    return params


def volume_components_to_frames_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeComponentsToFramesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def volume_components_to_frames_cargs(
    params: VolumeComponentsToFramesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-components-to-frames"
    ])
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    return cargs


def volume_components_to_frames_outputs(
    params: VolumeComponentsToFramesParamsDict,
    execution: Execution,
) -> VolumeComponentsToFramesOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeComponentsToFramesOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def volume_components_to_frames_execute(
    params: VolumeComponentsToFramesParamsDict,
    runner: Runner | None = None,
) -> VolumeComponentsToFramesOutputs:
    """
    CONVERT RGB/COMPLEX VOLUME TO FRAMES.
    
    RGB and complex datatypes are not always well supported, this command allows
    separating them into standard subvolumes for better support.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeComponentsToFramesOutputs`).
    """
    volume_components_to_frames_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_COMPONENTS_TO_FRAMES_METADATA)
    params = execution.params(params)
    cargs = volume_components_to_frames_cargs(params, execution)
    ret = volume_components_to_frames_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_components_to_frames(
    input_: InputPathType,
    output: str,
    runner: Runner | None = None,
) -> VolumeComponentsToFramesOutputs:
    """
    CONVERT RGB/COMPLEX VOLUME TO FRAMES.
    
    RGB and complex datatypes are not always well supported, this command allows
    separating them into standard subvolumes for better support.
    
    Args:
        input_: the RGB/complex-type volume.
        output: the input volume converted to multiple frames of scalar type.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeComponentsToFramesOutputs`).
    """
    params = volume_components_to_frames_params(
        input_=input_,
        output=output,
    )
    return volume_components_to_frames_execute(params, runner)


__all__ = [
    "VOLUME_COMPONENTS_TO_FRAMES_METADATA",
    "VolumeComponentsToFramesOutputs",
    "VolumeComponentsToFramesParamsDict",
    "VolumeComponentsToFramesParamsDictTagged",
    "volume_components_to_frames",
    "volume_components_to_frames_execute",
    "volume_components_to_frames_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

BORDER_MERGE_METADATA = Metadata(
    id="2b501fbb7f7c4f9515ae97ae16a101a97a54eaab.workbench",
    name="border-merge",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_BorderMergeUpToParamsDictNoTag = typing.TypedDict('_BorderMergeUpToParamsDictNoTag', {
    "last-border": str,
    "reverse": bool,
})
BorderMergeUpToParamsDictTagged = typing.TypedDict('BorderMergeUpToParamsDictTagged', {
    "@type": typing.Literal["up-to"],
    "last-border": str,
    "reverse": bool,
})
BorderMergeUpToParamsDict = _BorderMergeUpToParamsDictNoTag | BorderMergeUpToParamsDictTagged


_BorderMergeSelectParamsDictNoTag = typing.TypedDict('_BorderMergeSelectParamsDictNoTag', {
    "border": str,
    "up-to": typing.NotRequired[BorderMergeUpToParamsDict | None],
})
BorderMergeSelectParamsDictTagged = typing.TypedDict('BorderMergeSelectParamsDictTagged', {
    "@type": typing.Literal["select"],
    "border": str,
    "up-to": typing.NotRequired[BorderMergeUpToParamsDict | None],
})
BorderMergeSelectParamsDict = _BorderMergeSelectParamsDictNoTag | BorderMergeSelectParamsDictTagged


_BorderMergeBorderParamsDictNoTag = typing.TypedDict('_BorderMergeBorderParamsDictNoTag', {
    "border-file-in": InputPathType,
    "select": typing.NotRequired[list[BorderMergeSelectParamsDict] | None],
})
BorderMergeBorderParamsDictTagged = typing.TypedDict('BorderMergeBorderParamsDictTagged', {
    "@type": typing.Literal["border"],
    "border-file-in": InputPathType,
    "select": typing.NotRequired[list[BorderMergeSelectParamsDict] | None],
})
BorderMergeBorderParamsDict = _BorderMergeBorderParamsDictNoTag | BorderMergeBorderParamsDictTagged


_BorderMergeParamsDictNoTag = typing.TypedDict('_BorderMergeParamsDictNoTag', {
    "border-file-out": str,
    "border": typing.NotRequired[list[BorderMergeBorderParamsDict] | None],
})
BorderMergeParamsDictTagged = typing.TypedDict('BorderMergeParamsDictTagged', {
    "@type": typing.Literal["workbench/border-merge"],
    "border-file-out": str,
    "border": typing.NotRequired[list[BorderMergeBorderParamsDict] | None],
})
BorderMergeParamsDict = _BorderMergeParamsDictNoTag | BorderMergeParamsDictTagged


def border_merge_up_to(
    last_border: str,
    reverse: bool = False,
) -> BorderMergeUpToParamsDictTagged:
    """
    Build parameters.
    
    Args:
        last_border: the number or name of the last column to include.
        reverse: use the range in reverse order.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "up-to",
        "last-border": last_border,
        "reverse": reverse,
    }
    return params


def border_merge_up_to_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `BorderMergeUpToParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("last-border", None) is None:
        raise StyxValidationError("`last-border` must not be None")
    if not isinstance(params["last-border"], str):
        raise StyxValidationError(f'`last-border` has the wrong type: Received `{type(params.get("last-border", None))}` expected `str`')
    if params.get("reverse", False) is None:
        raise StyxValidationError("`reverse` must not be None")
    if not isinstance(params["reverse"], bool):
        raise StyxValidationError(f'`reverse` has the wrong type: Received `{type(params.get("reverse", False))}` expected `bool`')


def border_merge_up_to_cargs(
    params: BorderMergeUpToParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-up-to",
        params.get("last-border", None)
    ])
    if params.get("reverse", False):
        cargs.append("-reverse")
    return cargs


def border_merge_select(
    border: str,
    up_to: BorderMergeUpToParamsDict | None = None,
) -> BorderMergeSelectParamsDictTagged:
    """
    Build parameters.
    
    Args:
        border: the border number or name.
        up_to: use an inclusive range of borders.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "select",
        "border": border,
    }
    if up_to is not None:
        params["up-to"] = up_to
    return params


def border_merge_select_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `BorderMergeSelectParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("border", None) is None:
        raise StyxValidationError("`border` must not be None")
    if not isinstance(params["border"], str):
        raise StyxValidationError(f'`border` has the wrong type: Received `{type(params.get("border", None))}` expected `str`')
    if params.get("up-to", None) is not None:
        border_merge_up_to_validate(params["up-to"])


def border_merge_select_cargs(
    params: BorderMergeSelectParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-select",
        params.get("border", None),
        *(border_merge_up_to_cargs(params.get("up-to", None), execution) if (params.get("up-to", None) is not None) else [])
    ])
    return cargs


def border_merge_border(
    border_file_in: InputPathType,
    select_: list[BorderMergeSelectParamsDict] | None = None,
) -> BorderMergeBorderParamsDictTagged:
    """
    Build parameters.
    
    Args:
        border_file_in: a border file to use borders from.
        select_: select a single border to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "border",
        "border-file-in": border_file_in,
    }
    if select_ is not None:
        params["select"] = select_
    return params


def border_merge_border_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `BorderMergeBorderParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("border-file-in", None) is None:
        raise StyxValidationError("`border-file-in` must not be None")
    if not isinstance(params["border-file-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`border-file-in` has the wrong type: Received `{type(params.get("border-file-in", None))}` expected `InputPathType`')
    if params.get("select", None) is not None:
        if not isinstance(params["select"], list):
            raise StyxValidationError(f'`select` has the wrong type: Received `{type(params.get("select", None))}` expected `list[BorderMergeSelectParamsDict] | None`')
        for e in params["select"]:
            border_merge_select_validate(e)


def border_merge_border_cargs(
    params: BorderMergeBorderParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-border",
        execution.input_file(params.get("border-file-in", None)),
        *([a for c in [border_merge_select_cargs(s, execution) for s in params.get("select", None)] for a in c] if (params.get("select", None) is not None) else [])
    ])
    return cargs


class BorderMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `BorderMergeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    border_file_out: OutputPathType
    """the output border file"""


def border_merge_params(
    border_file_out: str,
    border: list[BorderMergeBorderParamsDict] | None = None,
) -> BorderMergeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        border_file_out: the output border file.
        border: specify an input border file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/border-merge",
        "border-file-out": border_file_out,
    }
    if border is not None:
        params["border"] = border
    return params


def border_merge_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `BorderMergeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("border-file-out", None) is None:
        raise StyxValidationError("`border-file-out` must not be None")
    if not isinstance(params["border-file-out"], str):
        raise StyxValidationError(f'`border-file-out` has the wrong type: Received `{type(params.get("border-file-out", None))}` expected `str`')
    if params.get("border", None) is not None:
        if not isinstance(params["border"], list):
            raise StyxValidationError(f'`border` has the wrong type: Received `{type(params.get("border", None))}` expected `list[BorderMergeBorderParamsDict] | None`')
        for e in params["border"]:
            border_merge_border_validate(e)


def border_merge_cargs(
    params: BorderMergeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-border-merge"
    ])
    cargs.append(params.get("border-file-out", None))
    if params.get("border", None) is not None:
        cargs.extend([a for c in [border_merge_border_cargs(s, execution) for s in params.get("border", None)] for a in c])
    return cargs


def border_merge_outputs(
    params: BorderMergeParamsDict,
    execution: Execution,
) -> BorderMergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = BorderMergeOutputs(
        root=execution.output_file("."),
        border_file_out=execution.output_file(params.get("border-file-out", None)),
    )
    return ret


def border_merge_execute(
    params: BorderMergeParamsDict,
    runner: Runner | None = None,
) -> BorderMergeOutputs:
    """
    MERGE BORDER FILES INTO A NEW FILE.
    
    Takes one or more border files and makes a new border file from the borders
    in them.
    
    Example: wb_command -border-merge out.border -border first.border -select 1
    -border second.border
    
    This example would take the first border from first.border, followed by all
    borders from second.border, and write these to out.border.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BorderMergeOutputs`).
    """
    border_merge_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(BORDER_MERGE_METADATA)
    params = execution.params(params)
    cargs = border_merge_cargs(params, execution)
    ret = border_merge_outputs(params, execution)
    execution.run(cargs)
    return ret


def border_merge(
    border_file_out: str,
    border: list[BorderMergeBorderParamsDict] | None = None,
    runner: Runner | None = None,
) -> BorderMergeOutputs:
    """
    MERGE BORDER FILES INTO A NEW FILE.
    
    Takes one or more border files and makes a new border file from the borders
    in them.
    
    Example: wb_command -border-merge out.border -border first.border -select 1
    -border second.border
    
    This example would take the first border from first.border, followed by all
    borders from second.border, and write these to out.border.
    
    Args:
        border_file_out: the output border file.
        border: specify an input border file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BorderMergeOutputs`).
    """
    params = border_merge_params(
        border_file_out=border_file_out,
        border=border,
    )
    return border_merge_execute(params, runner)


__all__ = [
    "BORDER_MERGE_METADATA",
    "BorderMergeBorderParamsDict",
    "BorderMergeBorderParamsDictTagged",
    "BorderMergeOutputs",
    "BorderMergeParamsDict",
    "BorderMergeParamsDictTagged",
    "BorderMergeSelectParamsDict",
    "BorderMergeSelectParamsDictTagged",
    "BorderMergeUpToParamsDict",
    "BorderMergeUpToParamsDictTagged",
    "border_merge",
    "border_merge_border",
    "border_merge_execute",
    "border_merge_params",
    "border_merge_select",
    "border_merge_up_to",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONVERT_MATRIX4_TO_WORKBENCH_SPARSE_METADATA = Metadata(
    id="bb2236b447f9edba4ba2e429ee0ed0348145ebe7.workbench",
    name="convert-matrix4-to-workbench-sparse",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDictNoTag = typing.TypedDict('_ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDictNoTag', {
    "cifti-template": InputPathType,
    "direction": str,
})
ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDictTagged = typing.TypedDict('ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDictTagged', {
    "@type": typing.Literal["volume-seeds"],
    "cifti-template": InputPathType,
    "direction": str,
})
ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDict = _ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDictNoTag | ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDictTagged


_ConvertMatrix4ToWorkbenchSparseParamsDictNoTag = typing.TypedDict('_ConvertMatrix4ToWorkbenchSparseParamsDictNoTag', {
    "matrix4_1": str,
    "matrix4_2": str,
    "matrix4_3": str,
    "orientation-file": InputPathType,
    "voxel-list": str,
    "wb-sparse-out": str,
    "volume-seeds": typing.NotRequired[ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDict | None],
    "seed-roi": typing.NotRequired[InputPathType | None],
})
ConvertMatrix4ToWorkbenchSparseParamsDictTagged = typing.TypedDict('ConvertMatrix4ToWorkbenchSparseParamsDictTagged', {
    "@type": typing.Literal["workbench/convert-matrix4-to-workbench-sparse"],
    "matrix4_1": str,
    "matrix4_2": str,
    "matrix4_3": str,
    "orientation-file": InputPathType,
    "voxel-list": str,
    "wb-sparse-out": str,
    "volume-seeds": typing.NotRequired[ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDict | None],
    "seed-roi": typing.NotRequired[InputPathType | None],
})
ConvertMatrix4ToWorkbenchSparseParamsDict = _ConvertMatrix4ToWorkbenchSparseParamsDictNoTag | ConvertMatrix4ToWorkbenchSparseParamsDictTagged


def convert_matrix4_to_workbench_sparse_volume_seeds(
    cifti_template: InputPathType,
    direction: str,
) -> ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_template: cifti file to use the volume mappings from.
        direction: dimension along the cifti file to take the mapping from, ROW\
            or COLUMN.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume-seeds",
        "cifti-template": cifti_template,
        "direction": direction,
    }
    return params


def convert_matrix4_to_workbench_sparse_volume_seeds_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-template", None) is None:
        raise StyxValidationError("`cifti-template` must not be None")
    if not isinstance(params["cifti-template"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-template` has the wrong type: Received `{type(params.get("cifti-template", None))}` expected `InputPathType`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')


def convert_matrix4_to_workbench_sparse_volume_seeds_cargs(
    params: ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume-seeds",
        execution.input_file(params.get("cifti-template", None)),
        params.get("direction", None)
    ])
    return cargs


class ConvertMatrix4ToWorkbenchSparseOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ConvertMatrix4ToWorkbenchSparseParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def convert_matrix4_to_workbench_sparse_params(
    matrix4_1: str,
    matrix4_2: str,
    matrix4_3: str,
    orientation_file: InputPathType,
    voxel_list: str,
    wb_sparse_out: str,
    volume_seeds: ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDict | None = None,
    seed_roi: InputPathType | None = None,
) -> ConvertMatrix4ToWorkbenchSparseParamsDictTagged:
    """
    Build parameters.
    
    Args:
        matrix4_1: the first matrix4 file.
        matrix4_2: the second matrix4 file.
        matrix4_3: the third matrix4 file.
        orientation_file: the .fiberTEMP.nii file this trajectory file applies\
            to.
        voxel_list: list of white matter voxel index triplets as used in the\
            trajectory matrix.
        wb_sparse_out: output - the output workbench sparse file.
        volume_seeds: specify the volume seed space.
        seed_roi: specify the surface seed space\
            \
            metric roi file of all vertices used in the seed space.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/convert-matrix4-to-workbench-sparse",
        "matrix4_1": matrix4_1,
        "matrix4_2": matrix4_2,
        "matrix4_3": matrix4_3,
        "orientation-file": orientation_file,
        "voxel-list": voxel_list,
        "wb-sparse-out": wb_sparse_out,
    }
    if volume_seeds is not None:
        params["volume-seeds"] = volume_seeds
    if seed_roi is not None:
        params["seed-roi"] = seed_roi
    return params


def convert_matrix4_to_workbench_sparse_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertMatrix4ToWorkbenchSparseParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("matrix4_1", None) is None:
        raise StyxValidationError("`matrix4_1` must not be None")
    if not isinstance(params["matrix4_1"], str):
        raise StyxValidationError(f'`matrix4_1` has the wrong type: Received `{type(params.get("matrix4_1", None))}` expected `str`')
    if params.get("matrix4_2", None) is None:
        raise StyxValidationError("`matrix4_2` must not be None")
    if not isinstance(params["matrix4_2"], str):
        raise StyxValidationError(f'`matrix4_2` has the wrong type: Received `{type(params.get("matrix4_2", None))}` expected `str`')
    if params.get("matrix4_3", None) is None:
        raise StyxValidationError("`matrix4_3` must not be None")
    if not isinstance(params["matrix4_3"], str):
        raise StyxValidationError(f'`matrix4_3` has the wrong type: Received `{type(params.get("matrix4_3", None))}` expected `str`')
    if params.get("orientation-file", None) is None:
        raise StyxValidationError("`orientation-file` must not be None")
    if not isinstance(params["orientation-file"], (pathlib.Path, str)):
        raise StyxValidationError(f'`orientation-file` has the wrong type: Received `{type(params.get("orientation-file", None))}` expected `InputPathType`')
    if params.get("voxel-list", None) is None:
        raise StyxValidationError("`voxel-list` must not be None")
    if not isinstance(params["voxel-list"], str):
        raise StyxValidationError(f'`voxel-list` has the wrong type: Received `{type(params.get("voxel-list", None))}` expected `str`')
    if params.get("wb-sparse-out", None) is None:
        raise StyxValidationError("`wb-sparse-out` must not be None")
    if not isinstance(params["wb-sparse-out"], str):
        raise StyxValidationError(f'`wb-sparse-out` has the wrong type: Received `{type(params.get("wb-sparse-out", None))}` expected `str`')
    if params.get("volume-seeds", None) is not None:
        convert_matrix4_to_workbench_sparse_volume_seeds_validate(params["volume-seeds"])
    if params.get("seed-roi", None) is not None:
        if not isinstance(params["seed-roi"], (pathlib.Path, str)):
            raise StyxValidationError(f'`seed-roi` has the wrong type: Received `{type(params.get("seed-roi", None))}` expected `InputPathType | None`')


def convert_matrix4_to_workbench_sparse_cargs(
    params: ConvertMatrix4ToWorkbenchSparseParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-convert-matrix4-to-workbench-sparse"
    ])
    cargs.append(params.get("matrix4_1", None))
    cargs.append(params.get("matrix4_2", None))
    cargs.append(params.get("matrix4_3", None))
    cargs.append(execution.input_file(params.get("orientation-file", None)))
    cargs.append(params.get("voxel-list", None))
    cargs.append(params.get("wb-sparse-out", None))
    if params.get("volume-seeds", None) is not None:
        cargs.extend(convert_matrix4_to_workbench_sparse_volume_seeds_cargs(params.get("volume-seeds", None), execution))
    if params.get("seed-roi", None) is not None:
        cargs.extend([
            "-surface-seeds",
            execution.input_file(params.get("seed-roi", None))
        ])
    return cargs


def convert_matrix4_to_workbench_sparse_outputs(
    params: ConvertMatrix4ToWorkbenchSparseParamsDict,
    execution: Execution,
) -> ConvertMatrix4ToWorkbenchSparseOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertMatrix4ToWorkbenchSparseOutputs(
        root=execution.output_file("."),
    )
    return ret


def convert_matrix4_to_workbench_sparse_execute(
    params: ConvertMatrix4ToWorkbenchSparseParamsDict,
    runner: Runner | None = None,
) -> ConvertMatrix4ToWorkbenchSparseOutputs:
    """
    CONVERT A 3-FILE MATRIX4 TO A WORKBENCH SPARSE FILE.
    
    Converts the matrix 4 output of probtrackx to workbench sparse file format.
    Exactly one of -surface-seeds and -volume-seeds must be specified.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertMatrix4ToWorkbenchSparseOutputs`).
    """
    convert_matrix4_to_workbench_sparse_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERT_MATRIX4_TO_WORKBENCH_SPARSE_METADATA)
    params = execution.params(params)
    cargs = convert_matrix4_to_workbench_sparse_cargs(params, execution)
    ret = convert_matrix4_to_workbench_sparse_outputs(params, execution)
    execution.run(cargs)
    return ret


def convert_matrix4_to_workbench_sparse(
    matrix4_1: str,
    matrix4_2: str,
    matrix4_3: str,
    orientation_file: InputPathType,
    voxel_list: str,
    wb_sparse_out: str,
    volume_seeds: ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDict | None = None,
    seed_roi: InputPathType | None = None,
    runner: Runner | None = None,
) -> ConvertMatrix4ToWorkbenchSparseOutputs:
    """
    CONVERT A 3-FILE MATRIX4 TO A WORKBENCH SPARSE FILE.
    
    Converts the matrix 4 output of probtrackx to workbench sparse file format.
    Exactly one of -surface-seeds and -volume-seeds must be specified.
    
    Args:
        matrix4_1: the first matrix4 file.
        matrix4_2: the second matrix4 file.
        matrix4_3: the third matrix4 file.
        orientation_file: the .fiberTEMP.nii file this trajectory file applies\
            to.
        voxel_list: list of white matter voxel index triplets as used in the\
            trajectory matrix.
        wb_sparse_out: output - the output workbench sparse file.
        volume_seeds: specify the volume seed space.
        seed_roi: specify the surface seed space\
            \
            metric roi file of all vertices used in the seed space.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertMatrix4ToWorkbenchSparseOutputs`).
    """
    params = convert_matrix4_to_workbench_sparse_params(
        matrix4_1=matrix4_1,
        matrix4_2=matrix4_2,
        matrix4_3=matrix4_3,
        orientation_file=orientation_file,
        voxel_list=voxel_list,
        wb_sparse_out=wb_sparse_out,
        volume_seeds=volume_seeds,
        seed_roi=seed_roi,
    )
    return convert_matrix4_to_workbench_sparse_execute(params, runner)


__all__ = [
    "CONVERT_MATRIX4_TO_WORKBENCH_SPARSE_METADATA",
    "ConvertMatrix4ToWorkbenchSparseOutputs",
    "ConvertMatrix4ToWorkbenchSparseParamsDict",
    "ConvertMatrix4ToWorkbenchSparseParamsDictTagged",
    "ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDict",
    "ConvertMatrix4ToWorkbenchSparseVolumeSeedsParamsDictTagged",
    "convert_matrix4_to_workbench_sparse",
    "convert_matrix4_to_workbench_sparse_execute",
    "convert_matrix4_to_workbench_sparse_params",
    "convert_matrix4_to_workbench_sparse_volume_seeds",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_LABEL_TO_ROI_METADATA = Metadata(
    id="f849408f8ec3fabb1a2cc5fda20d4180749ba643.workbench",
    name="volume-label-to-roi",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeLabelToRoiParamsDictNoTag = typing.TypedDict('_VolumeLabelToRoiParamsDictNoTag', {
    "label-in": InputPathType,
    "volume-out": str,
    "map": typing.NotRequired[str | None],
    "label-key": typing.NotRequired[int | None],
    "label-name": typing.NotRequired[str | None],
})
VolumeLabelToRoiParamsDictTagged = typing.TypedDict('VolumeLabelToRoiParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-label-to-roi"],
    "label-in": InputPathType,
    "volume-out": str,
    "map": typing.NotRequired[str | None],
    "label-key": typing.NotRequired[int | None],
    "label-name": typing.NotRequired[str | None],
})
VolumeLabelToRoiParamsDict = _VolumeLabelToRoiParamsDictNoTag | VolumeLabelToRoiParamsDictTagged


class VolumeLabelToRoiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeLabelToRoiParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume file"""


def volume_label_to_roi_params(
    label_in: InputPathType,
    volume_out: str,
    map_: str | None = None,
    label_key: int | None = None,
    label_name: str | None = None,
) -> VolumeLabelToRoiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_in: the input volume label file.
        volume_out: the output volume file.
        map_: select a single label map to use\
            \
            the map number or name.
        label_key: select label by key\
            \
            the label key that you want an roi of.
        label_name: select label by name\
            \
            the label name that you want an roi of.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-label-to-roi",
        "label-in": label_in,
        "volume-out": volume_out,
    }
    if map_ is not None:
        params["map"] = map_
    if label_key is not None:
        params["label-key"] = label_key
    if label_name is not None:
        params["label-name"] = label_name
    return params


def volume_label_to_roi_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeLabelToRoiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-in", None) is None:
        raise StyxValidationError("`label-in` must not be None")
    if not isinstance(params["label-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-in` has the wrong type: Received `{type(params.get("label-in", None))}` expected `InputPathType`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("map", None) is not None:
        if not isinstance(params["map"], str):
            raise StyxValidationError(f'`map` has the wrong type: Received `{type(params.get("map", None))}` expected `str | None`')
    if params.get("label-key", None) is not None:
        if not isinstance(params["label-key"], int):
            raise StyxValidationError(f'`label-key` has the wrong type: Received `{type(params.get("label-key", None))}` expected `int | None`')
    if params.get("label-name", None) is not None:
        if not isinstance(params["label-name"], str):
            raise StyxValidationError(f'`label-name` has the wrong type: Received `{type(params.get("label-name", None))}` expected `str | None`')


def volume_label_to_roi_cargs(
    params: VolumeLabelToRoiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-label-to-roi"
    ])
    cargs.append(execution.input_file(params.get("label-in", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("map", None) is not None:
        cargs.extend([
            "-map",
            params.get("map", None)
        ])
    if params.get("label-key", None) is not None:
        cargs.extend([
            "-key",
            str(params.get("label-key", None))
        ])
    if params.get("label-name", None) is not None:
        cargs.extend([
            "-name",
            params.get("label-name", None)
        ])
    return cargs


def volume_label_to_roi_outputs(
    params: VolumeLabelToRoiParamsDict,
    execution: Execution,
) -> VolumeLabelToRoiOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeLabelToRoiOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_label_to_roi_execute(
    params: VolumeLabelToRoiParamsDict,
    runner: Runner | None = None,
) -> VolumeLabelToRoiOutputs:
    """
    MAKE A VOLUME LABEL INTO AN ROI VOLUME.
    
    For each map in <label-in>, a map is created in <volume-out> where all
    locations labeled with <label-name> or with a key of <label-key> are given a
    value of 1, and all other locations are given 0. Exactly one of -name and
    -key must be specified. Specify -map to use only one map from <label-in>.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeLabelToRoiOutputs`).
    """
    volume_label_to_roi_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_LABEL_TO_ROI_METADATA)
    params = execution.params(params)
    cargs = volume_label_to_roi_cargs(params, execution)
    ret = volume_label_to_roi_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_label_to_roi(
    label_in: InputPathType,
    volume_out: str,
    map_: str | None = None,
    label_key: int | None = None,
    label_name: str | None = None,
    runner: Runner | None = None,
) -> VolumeLabelToRoiOutputs:
    """
    MAKE A VOLUME LABEL INTO AN ROI VOLUME.
    
    For each map in <label-in>, a map is created in <volume-out> where all
    locations labeled with <label-name> or with a key of <label-key> are given a
    value of 1, and all other locations are given 0. Exactly one of -name and
    -key must be specified. Specify -map to use only one map from <label-in>.
    
    Args:
        label_in: the input volume label file.
        volume_out: the output volume file.
        map_: select a single label map to use\
            \
            the map number or name.
        label_key: select label by key\
            \
            the label key that you want an roi of.
        label_name: select label by name\
            \
            the label name that you want an roi of.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeLabelToRoiOutputs`).
    """
    params = volume_label_to_roi_params(
        label_in=label_in,
        volume_out=volume_out,
        map_=map_,
        label_key=label_key,
        label_name=label_name,
    )
    return volume_label_to_roi_execute(params, runner)


__all__ = [
    "VOLUME_LABEL_TO_ROI_METADATA",
    "VolumeLabelToRoiOutputs",
    "VolumeLabelToRoiParamsDict",
    "VolumeLabelToRoiParamsDictTagged",
    "volume_label_to_roi",
    "volume_label_to_roi_execute",
    "volume_label_to_roi_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_CURVATURE_METADATA = Metadata(
    id="d5eee7e4c555970158a785c68fecd07f7b6649cb.workbench",
    name="surface-curvature",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceCurvatureMeanParamsDictNoTag = typing.TypedDict('_SurfaceCurvatureMeanParamsDictNoTag', {
    "mean-out": str,
})
SurfaceCurvatureMeanParamsDictTagged = typing.TypedDict('SurfaceCurvatureMeanParamsDictTagged', {
    "@type": typing.Literal["mean"],
    "mean-out": str,
})
SurfaceCurvatureMeanParamsDict = _SurfaceCurvatureMeanParamsDictNoTag | SurfaceCurvatureMeanParamsDictTagged


_SurfaceCurvatureGaussParamsDictNoTag = typing.TypedDict('_SurfaceCurvatureGaussParamsDictNoTag', {
    "gauss-out": str,
})
SurfaceCurvatureGaussParamsDictTagged = typing.TypedDict('SurfaceCurvatureGaussParamsDictTagged', {
    "@type": typing.Literal["gauss"],
    "gauss-out": str,
})
SurfaceCurvatureGaussParamsDict = _SurfaceCurvatureGaussParamsDictNoTag | SurfaceCurvatureGaussParamsDictTagged


_SurfaceCurvatureParamsDictNoTag = typing.TypedDict('_SurfaceCurvatureParamsDictNoTag', {
    "surface": InputPathType,
    "mean": typing.NotRequired[SurfaceCurvatureMeanParamsDict | None],
    "gauss": typing.NotRequired[SurfaceCurvatureGaussParamsDict | None],
})
SurfaceCurvatureParamsDictTagged = typing.TypedDict('SurfaceCurvatureParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-curvature"],
    "surface": InputPathType,
    "mean": typing.NotRequired[SurfaceCurvatureMeanParamsDict | None],
    "gauss": typing.NotRequired[SurfaceCurvatureGaussParamsDict | None],
})
SurfaceCurvatureParamsDict = _SurfaceCurvatureParamsDictNoTag | SurfaceCurvatureParamsDictTagged


class SurfaceCurvatureMeanOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceCurvatureMeanParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    mean_out: OutputPathType
    """mean curvature metric"""


def surface_curvature_mean(
    mean_out: str,
) -> SurfaceCurvatureMeanParamsDictTagged:
    """
    Build parameters.
    
    Args:
        mean_out: mean curvature metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mean",
        "mean-out": mean_out,
    }
    return params


def surface_curvature_mean_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceCurvatureMeanParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("mean-out", None) is None:
        raise StyxValidationError("`mean-out` must not be None")
    if not isinstance(params["mean-out"], str):
        raise StyxValidationError(f'`mean-out` has the wrong type: Received `{type(params.get("mean-out", None))}` expected `str`')


def surface_curvature_mean_cargs(
    params: SurfaceCurvatureMeanParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-mean",
        params.get("mean-out", None)
    ])
    return cargs


def surface_curvature_mean_outputs(
    params: SurfaceCurvatureMeanParamsDict,
    execution: Execution,
) -> SurfaceCurvatureMeanOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceCurvatureMeanOutputs(
        root=execution.output_file("."),
        mean_out=execution.output_file(params.get("mean-out", None)),
    )
    return ret


class SurfaceCurvatureGaussOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceCurvatureGaussParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    gauss_out: OutputPathType
    """gaussian curvature metric"""


def surface_curvature_gauss(
    gauss_out: str,
) -> SurfaceCurvatureGaussParamsDictTagged:
    """
    Build parameters.
    
    Args:
        gauss_out: gaussian curvature metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "gauss",
        "gauss-out": gauss_out,
    }
    return params


def surface_curvature_gauss_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceCurvatureGaussParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("gauss-out", None) is None:
        raise StyxValidationError("`gauss-out` must not be None")
    if not isinstance(params["gauss-out"], str):
        raise StyxValidationError(f'`gauss-out` has the wrong type: Received `{type(params.get("gauss-out", None))}` expected `str`')


def surface_curvature_gauss_cargs(
    params: SurfaceCurvatureGaussParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-gauss",
        params.get("gauss-out", None)
    ])
    return cargs


def surface_curvature_gauss_outputs(
    params: SurfaceCurvatureGaussParamsDict,
    execution: Execution,
) -> SurfaceCurvatureGaussOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceCurvatureGaussOutputs(
        root=execution.output_file("."),
        gauss_out=execution.output_file(params.get("gauss-out", None)),
    )
    return ret


class SurfaceCurvatureOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceCurvatureParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    mean: SurfaceCurvatureMeanOutputs | None
    """Outputs from `surface_curvature_mean_outputs`."""
    gauss: SurfaceCurvatureGaussOutputs | None
    """Outputs from `surface_curvature_gauss_outputs`."""


def surface_curvature_params(
    surface: InputPathType,
    mean: SurfaceCurvatureMeanParamsDict | None = None,
    gauss: SurfaceCurvatureGaussParamsDict | None = None,
) -> SurfaceCurvatureParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to compute the curvature of.
        mean: output mean curvature.
        gauss: output gaussian curvature.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-curvature",
        "surface": surface,
    }
    if mean is not None:
        params["mean"] = mean
    if gauss is not None:
        params["gauss"] = gauss
    return params


def surface_curvature_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceCurvatureParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("mean", None) is not None:
        surface_curvature_mean_validate(params["mean"])
    if params.get("gauss", None) is not None:
        surface_curvature_gauss_validate(params["gauss"])


def surface_curvature_cargs(
    params: SurfaceCurvatureParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-curvature"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("mean", None) is not None or params.get("gauss", None) is not None:
        cargs.extend([
            *(surface_curvature_mean_cargs(params.get("mean", None), execution) if (params.get("mean", None) is not None) else []),
            *(surface_curvature_gauss_cargs(params.get("gauss", None), execution) if (params.get("gauss", None) is not None) else [])
        ])
    return cargs


def surface_curvature_outputs(
    params: SurfaceCurvatureParamsDict,
    execution: Execution,
) -> SurfaceCurvatureOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceCurvatureOutputs(
        root=execution.output_file("."),
        mean=surface_curvature_mean_outputs(params.get("mean"), execution) if params.get("mean") else None,
        gauss=surface_curvature_gauss_outputs(params.get("gauss"), execution) if params.get("gauss") else None,
    )
    return ret


def surface_curvature_execute(
    params: SurfaceCurvatureParamsDict,
    runner: Runner | None = None,
) -> SurfaceCurvatureOutputs:
    """
    CALCULATE CURVATURE OF SURFACE.
    
    Compute the curvature of the surface, using the method from:
    Interactive Texture Mapping by J. Maillot, Yahia, and Verroust, 1993.
    ACM-0-98791-601-8/93/008.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceCurvatureOutputs`).
    """
    surface_curvature_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_CURVATURE_METADATA)
    params = execution.params(params)
    cargs = surface_curvature_cargs(params, execution)
    ret = surface_curvature_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_curvature(
    surface: InputPathType,
    mean: SurfaceCurvatureMeanParamsDict | None = None,
    gauss: SurfaceCurvatureGaussParamsDict | None = None,
    runner: Runner | None = None,
) -> SurfaceCurvatureOutputs:
    """
    CALCULATE CURVATURE OF SURFACE.
    
    Compute the curvature of the surface, using the method from:
    Interactive Texture Mapping by J. Maillot, Yahia, and Verroust, 1993.
    ACM-0-98791-601-8/93/008.
    
    Args:
        surface: the surface to compute the curvature of.
        mean: output mean curvature.
        gauss: output gaussian curvature.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceCurvatureOutputs`).
    """
    params = surface_curvature_params(
        surface=surface,
        mean=mean,
        gauss=gauss,
    )
    return surface_curvature_execute(params, runner)


__all__ = [
    "SURFACE_CURVATURE_METADATA",
    "SurfaceCurvatureGaussOutputs",
    "SurfaceCurvatureGaussParamsDict",
    "SurfaceCurvatureGaussParamsDictTagged",
    "SurfaceCurvatureMeanOutputs",
    "SurfaceCurvatureMeanParamsDict",
    "SurfaceCurvatureMeanParamsDictTagged",
    "SurfaceCurvatureOutputs",
    "SurfaceCurvatureParamsDict",
    "SurfaceCurvatureParamsDictTagged",
    "surface_curvature",
    "surface_curvature_execute",
    "surface_curvature_gauss",
    "surface_curvature_mean",
    "surface_curvature_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_ESTIMATE_FWHM_METADATA = Metadata(
    id="bfd040551c3a70785cb9174fd0ff584bb1326d8a.workbench",
    name="cifti-estimate-fwhm",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiEstimateFwhmSurfaceParamsDictNoTag = typing.TypedDict('_CiftiEstimateFwhmSurfaceParamsDictNoTag', {
    "structure": str,
    "surface": InputPathType,
})
CiftiEstimateFwhmSurfaceParamsDictTagged = typing.TypedDict('CiftiEstimateFwhmSurfaceParamsDictTagged', {
    "@type": typing.Literal["surface"],
    "structure": str,
    "surface": InputPathType,
})
CiftiEstimateFwhmSurfaceParamsDict = _CiftiEstimateFwhmSurfaceParamsDictNoTag | CiftiEstimateFwhmSurfaceParamsDictTagged


_CiftiEstimateFwhmParamsDictNoTag = typing.TypedDict('_CiftiEstimateFwhmParamsDictNoTag', {
    "cifti": InputPathType,
    "surface": typing.NotRequired[list[CiftiEstimateFwhmSurfaceParamsDict] | None],
    "demean": typing.NotRequired[bool | None],
    "column": typing.NotRequired[int | None],
    "merged-volume": bool,
})
CiftiEstimateFwhmParamsDictTagged = typing.TypedDict('CiftiEstimateFwhmParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-estimate-fwhm"],
    "cifti": InputPathType,
    "surface": typing.NotRequired[list[CiftiEstimateFwhmSurfaceParamsDict] | None],
    "demean": typing.NotRequired[bool | None],
    "column": typing.NotRequired[int | None],
    "merged-volume": bool,
})
CiftiEstimateFwhmParamsDict = _CiftiEstimateFwhmParamsDictNoTag | CiftiEstimateFwhmParamsDictTagged


def cifti_estimate_fwhm_surface(
    structure: str,
    surface: InputPathType,
) -> CiftiEstimateFwhmSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: what structure to use this surface for.
        surface: the surface file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "surface",
        "structure": structure,
        "surface": surface,
    }
    return params


def cifti_estimate_fwhm_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiEstimateFwhmSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')


def cifti_estimate_fwhm_surface_cargs(
    params: CiftiEstimateFwhmSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-surface",
        params.get("structure", None),
        execution.input_file(params.get("surface", None))
    ])
    return cargs


class CiftiEstimateFwhmOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiEstimateFwhmParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def cifti_estimate_fwhm_params(
    cifti: InputPathType,
    surface: list[CiftiEstimateFwhmSurfaceParamsDict] | None = None,
    demean: bool | None = None,
    column: int | None = None,
    merged_volume: bool = False,
) -> CiftiEstimateFwhmParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti: the input cifti file.
        surface: specify an input surface.
        demean: estimate for the whole file at once, not each column separately\
            \
            subtract the mean image before estimating smoothness.
        column: only output estimates for one column\
            \
            the column number.
        merged_volume: treat volume components as if they were a single\
            component.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-estimate-fwhm",
        "cifti": cifti,
        "merged-volume": merged_volume,
    }
    if surface is not None:
        params["surface"] = surface
    if demean is not None:
        params["demean"] = demean
    if column is not None:
        params["column"] = column
    return params


def cifti_estimate_fwhm_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiEstimateFwhmParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti", None) is None:
        raise StyxValidationError("`cifti` must not be None")
    if not isinstance(params["cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `InputPathType`')
    if params.get("surface", None) is not None:
        if not isinstance(params["surface"], list):
            raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `list[CiftiEstimateFwhmSurfaceParamsDict] | None`')
        for e in params["surface"]:
            cifti_estimate_fwhm_surface_validate(e)
    if params.get("demean", None) is not None:
        if not isinstance(params["demean"], bool):
            raise StyxValidationError(f'`demean` has the wrong type: Received `{type(params.get("demean", None))}` expected `bool | None`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], int):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `int | None`')
    if params.get("merged-volume", False) is None:
        raise StyxValidationError("`merged-volume` must not be None")
    if not isinstance(params["merged-volume"], bool):
        raise StyxValidationError(f'`merged-volume` has the wrong type: Received `{type(params.get("merged-volume", False))}` expected `bool`')


def cifti_estimate_fwhm_cargs(
    params: CiftiEstimateFwhmParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-estimate-fwhm"
    ])
    cargs.append(execution.input_file(params.get("cifti", None)))
    if params.get("surface", None) is not None:
        cargs.extend([a for c in [cifti_estimate_fwhm_surface_cargs(s, execution) for s in params.get("surface", None)] for a in c])
    if params.get("demean", None) is not None:
        cargs.extend([
            "-whole-file",
            "-demean"
        ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            str(params.get("column", None))
        ])
    if params.get("merged-volume", False):
        cargs.append("-merged-volume")
    return cargs


def cifti_estimate_fwhm_outputs(
    params: CiftiEstimateFwhmParamsDict,
    execution: Execution,
) -> CiftiEstimateFwhmOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiEstimateFwhmOutputs(
        root=execution.output_file("."),
    )
    return ret


def cifti_estimate_fwhm_execute(
    params: CiftiEstimateFwhmParamsDict,
    runner: Runner | None = None,
) -> CiftiEstimateFwhmOutputs:
    """
    ESTIMATE FWHM SMOOTHNESS OF A CIFTI FILE.
    
    Estimate the smoothness of the components of the cifti file, printing the
    estimates to standard output. If -merged-volume is used, all voxels are used
    as a single component, rather than separated by structure.
    
    <structure> must be one of the following:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiEstimateFwhmOutputs`).
    """
    cifti_estimate_fwhm_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_ESTIMATE_FWHM_METADATA)
    params = execution.params(params)
    cargs = cifti_estimate_fwhm_cargs(params, execution)
    ret = cifti_estimate_fwhm_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_estimate_fwhm(
    cifti: InputPathType,
    surface: list[CiftiEstimateFwhmSurfaceParamsDict] | None = None,
    demean: bool | None = None,
    column: int | None = None,
    merged_volume: bool = False,
    runner: Runner | None = None,
) -> CiftiEstimateFwhmOutputs:
    """
    ESTIMATE FWHM SMOOTHNESS OF A CIFTI FILE.
    
    Estimate the smoothness of the components of the cifti file, printing the
    estimates to standard output. If -merged-volume is used, all voxels are used
    as a single component, rather than separated by structure.
    
    <structure> must be one of the following:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        cifti: the input cifti file.
        surface: specify an input surface.
        demean: estimate for the whole file at once, not each column separately\
            \
            subtract the mean image before estimating smoothness.
        column: only output estimates for one column\
            \
            the column number.
        merged_volume: treat volume components as if they were a single\
            component.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiEstimateFwhmOutputs`).
    """
    params = cifti_estimate_fwhm_params(
        cifti=cifti,
        surface=surface,
        demean=demean,
        column=column,
        merged_volume=merged_volume,
    )
    return cifti_estimate_fwhm_execute(params, runner)


__all__ = [
    "CIFTI_ESTIMATE_FWHM_METADATA",
    "CiftiEstimateFwhmOutputs",
    "CiftiEstimateFwhmParamsDict",
    "CiftiEstimateFwhmParamsDictTagged",
    "CiftiEstimateFwhmSurfaceParamsDict",
    "CiftiEstimateFwhmSurfaceParamsDictTagged",
    "cifti_estimate_fwhm",
    "cifti_estimate_fwhm_execute",
    "cifti_estimate_fwhm_params",
    "cifti_estimate_fwhm_surface",
]
